/**************************************************************************//**
 * @file     startup_n32wb031.S
 * @brief    CMSIS-Core(M) Device Startup File for Nations N32WB03x Device
 * @version  V0.1.0
 * @date     20. Oct. 2023
 ******************************************************************************/
/*
 * Copyright (c) 2009-2020 Arm Limited. All rights reserved.
 * Copyright (c) 2023 Yilin Sun. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

                .syntax  unified
                .arch    armv6-m

                .section .vectors
                .align   2
                .globl   __Vectors
                .globl   __Vectors_End
                .globl   __Vectors_Size
__Vectors:
                .long    __StackTop                         /*     Top of Stack */
                .long    Reset_Handler                      /*     Reset Handler */
                .long    NMI_Handler                        /* -14 NMI Handler */
                .long    HardFault_Handler                  /* -13 Hard Fault Handler */
                .long    0                                  /*     Reserved */
                .long    0                                  /*     Reserved */
                .long    0                                  /*     Reserved */
                .long    0                                  /*     Reserved */
                .long    0                                  /*     Reserved */
                .long    0                                  /*     Reserved */
                .long    0                                  /*     Reserved */
                .long    SVC_Handler                        /*  -5 SVCall Handler */
                .long    0                                  /*     Reserved */
                .long    0                                  /*     Reserved */
                .long    PendSV_Handler                     /*  -2 PendSV Handler */
                .long    SysTick_Handler                    /*  -1 SysTick Handler */

                /* Interrupts */
                .long    WWDG_IRQHandler                    /*   0 Window Watchdog              */
                .long    BLE_SW_IRQHandler                  /*   1 BLE_SW interrupt             */
                .long    RTC_IRQHandler                     /*   2 RTC interrupt(through EXTI line 17/19/20 interrupt) */
                .long    BLE_HSLOT_IRQHandler               /*   3 BLE_HSLOT                    */
                .long    FLASH_IRQHandler                   /*   4 Flash                        */
                .long    RCC_IRQHandler                     /*   5 RCC                          */
                .long    EXTI0_1_IRQHandler                 /*   6 EXTI Line 0.1                */
                .long    EXTI2_3_IRQHandler                 /*   7 EXTI Line 2.3                */
                .long    EXTI4_12_IRQHandler                /*   8 EXTI Line 4..12              */
                .long    BLE_FINETGT_IRQHandler             /*   9 BLE_FINETGT                  */
                .long    BLE_FIFO_IRQHandler                /*  10 BLE_FIFO                     */
                .long    DMA_Channel1_2_3_4_IRQHandler      /*  11 DMA Channel 1,2,3,4          */
                .long    DMA_Channel5_IRQHandler            /*  12 DMA Channel 5,6,7,8          */
                .long    TIM1_BRK_UP_TRG_COM_IRQHandler     /*  13 TIM1 Break/Update/Trigger and Commutation */
                .long    TIM1_CC_IRQHandler                 /*  14 TIM1 Capture Compare         */
                .long    RESERVED_IRQHandler                /*  15 Reserved (BLE Switch)        */
                .long    TIM3_IRQHandler                    /*  16 TIM3                         */
                .long    BLE_ERROR_IRQHandler               /*  17 BLE_ERROR                    */
                .long    BLE_CRYPT_IRQHandler               /*  18 BLE_CYPT_IRQHandler          */
                .long    BLE_TIMESTAMP_TGT1_IRQHandler      /*  19 BLE_TIMESTAMP_TGT1           */
                .long    TIM6_IRQHandler                    /*  20 TIM6                         */
                .long    ADC_IRQHandler                     /*  21 ADC                          */
                .long    SPI2_IRQHandler                    /*  22 SPI2                         */
                .long    I2C1_IRQHandler                    /*  23 I2C1                         */
                .long    BLE_TIMESTAMP_TGT2_IRQHandler      /*  24 BLE_TIMESTAMP_TGT12          */
                .long    SPI1_IRQHandler                    /*  25 SPI1                         */
                .long    BLE_SLP_IRQHandler                 /*  26 BLE_SLP                      */
                .long    KEYSCAN_IRQHandler                 /*  27 KEYSCAN                      */
                .long    USART1_IRQHandler                  /*  28 USART1                       */
                .long    LPUART1_IRQHandler                 /*  29 LPUART1                      */
                .long    USART2_IRQHandler                  /*  30 USART2                       */
                .long    IRC_IRQHandler                     /*  31 IRC                          */

__Vectors_End:
                .equ     __Vectors_Size, __Vectors_End - __Vectors
                .size    __Vectors, . - __Vectors


                .thumb
                .section .text
                .align   2

                .thumb_func
                .type    Reset_Handler, %function
                .globl   Reset_Handler
                .fnstart
Reset_Handler:
                bl       SystemInit

                ldr      r4, =__copy_table_start__
                ldr      r5, =__copy_table_end__

.L_loop0:
                cmp      r4, r5
                bge      .L_loop0_done
                ldr      r1, [r4]                /* source address */
                ldr      r2, [r4, #4]            /* destination address */
                ldr      r3, [r4, #8]            /* word count */
                lsls     r3, r3, #2              /* byte count */

.L_loop0_0:
                subs     r3, #4                  /* decrement byte count */
                blt      .L_loop0_0_done
                ldr      r0, [r1, r3]
                str      r0, [r2, r3]
                b        .L_loop0_0

.L_loop0_0_done:
                adds     r4, #12
                b        .L_loop0

.L_loop0_done:

                ldr      r3, =__zero_table_start__
                ldr      r4, =__zero_table_end__

.L_loop2:
                cmp      r3, r4
                bge      .L_loop2_done
                ldr      r1, [r3]                /* destination address */
                ldr      r2, [r3, #4]            /* word count */
                lsls     r2, r2, #2              /* byte count */
                movs     r0, 0

.L_loop2_0:
                subs     r2, #4                  /* decrement byte count */
                blt      .L_loop2_0_done
                str      r0, [r1, r2]
                b        .L_loop2_0
.L_loop2_0_done:

                adds     r3, #8
                b        .L_loop2
.L_loop2_done:

                bl       _start

                .fnend
                .size    Reset_Handler, . - Reset_Handler

/* The default macro is not used for HardFault_Handler
 * because this results in a poor debug illusion.
 */
                .thumb_func
                .type    HardFault_Handler, %function
                .weak    HardFault_Handler
                .fnstart
HardFault_Handler:
                b        .
                .fnend
                .size    HardFault_Handler, . - HardFault_Handler

                .thumb_func
                .type    Default_Handler, %function
                .weak    Default_Handler
                .fnstart
Default_Handler:
                b        .
                .fnend
                .size    Default_Handler, . - Default_Handler

/* Macro to define default exception/interrupt handlers.
 * Default handler are weak symbols with an endless loop.
 * They can be overwritten by real handlers.
 */
                .macro   Set_Default_Handler  Handler_Name
                .weak    \Handler_Name
                .set     \Handler_Name, Default_Handler
                .endm


/* Default exception/interrupt handler */

                Set_Default_Handler  NMI_Handler
                Set_Default_Handler  SVC_Handler
                Set_Default_Handler  PendSV_Handler
                Set_Default_Handler  SysTick_Handler

                Set_Default_Handler  WWDG_IRQHandler
                Set_Default_Handler  BLE_SW_IRQHandler
                Set_Default_Handler  RTC_IRQHandler
                Set_Default_Handler  BLE_HSLOT_IRQHandler
                Set_Default_Handler  FLASH_IRQHandler
                Set_Default_Handler  RCC_IRQHandler
                Set_Default_Handler  EXTI0_1_IRQHandler
                Set_Default_Handler  EXTI2_3_IRQHandler
                Set_Default_Handler  EXTI4_12_IRQHandler
                Set_Default_Handler  BLE_FINETGT_IRQHandler
                Set_Default_Handler  BLE_FIFO_IRQHandler
                Set_Default_Handler  DMA_Channel1_2_3_4_IRQHandler
                Set_Default_Handler  DMA_Channel5_IRQHandler
                Set_Default_Handler  TIM1_BRK_UP_TRG_COM_IRQHandler
                Set_Default_Handler  TIM1_CC_IRQHandler
                Set_Default_Handler  RESERVED_IRQHandler
                Set_Default_Handler  TIM3_IRQHandler
                Set_Default_Handler  BLE_ERROR_IRQHandler
                Set_Default_Handler  BLE_CRYPT_IRQHandler
                Set_Default_Handler  BLE_TIMESTAMP_TGT1_IRQHandler
                Set_Default_Handler  TIM6_IRQHandler
                Set_Default_Handler  ADC_IRQHandler
                Set_Default_Handler  SPI2_IRQHandler
                Set_Default_Handler  I2C1_IRQHandler
                Set_Default_Handler  BLE_TIMESTAMP_TGT2_IRQHandler
                Set_Default_Handler  SPI1_IRQHandler
                Set_Default_Handler  BLE_SLP_IRQHandler
                Set_Default_Handler  KEYSCAN_IRQHandler
                Set_Default_Handler  USART1_IRQHandler
                Set_Default_Handler  LPUART1_IRQHandler
                Set_Default_Handler  USART2_IRQHandler
                Set_Default_Handler  IRC_IRQHandler

                .end